<?php

namespace Bydedi\Tracking;

use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\Request;
use Query;
use SiteSQL;
use Commande;
use Panier;

/**
 * Class Tracking
 * @package Bydedi\Tracking
 */
class Tracking
{
    /**
     * @var array|mixed
     */
    protected $trackers = array();

    /**
     * @var Request
     */
    protected $request;

    /**
     * Tracking constructor.
     * @param Request|null $request
     */
    public function __construct(Request $request = null)
    {
        if (is_null($request) == false) {
            $this->setRequest($request);
        }
    }

    /**
     * @param Request $request
     * @return $this
     */
    public function setRequest(Request $request)
    {
        $this->request = $request;
        return $this;
    }

    /**
     * @param $tracker
     */
    public function addTracker($tracker)
    {
        $this->trackers[] = $tracker;
    }

    /**
     * @param $action
     * @param array $param
     * @param null $template
     * @return array
     */
    public function get($action, $param = array(), &$template = null)
    {
        if (method_exists($this, $action)) {
            list($action, $param['extra']) = $this->$action($param);
        }

        $param['template'] = &$template;
        $result = array();
        foreach ($this->trackers as $class) {
            if (method_exists($class, $action)) {
                $result[$class] = call_user_func_array(array($class, $action), array($this->request, $param));
            }
        }

        return $result;
    }

    public function purchase($param = array())
    {

        if (empty($param['commande_id'])) {
            trigger_error('tracking : no commande_id', E_USER_NOTICE);
            return false;
        }

        // recuperation des données
        $commande = new Commande($param['commande_id']);

        // preparation des données
        $info = $commande->toArray();
        $info['produits'] = $commande->getProduit();

        // envoi des données
        return array('purchase', $info);
    }

    public function customer($param = array())
    {
        $session = $this->request->getSession();

        $infos = array();

        if ($session->has('client_id')) {
            $client = new \Client($session->get('client_id'));
            $infos = array_merge($infos, $client->toArray());
        }

        return array('customer', $infos);
    }

    public function checkout($param = array())
    {
        return array('checkout', $this->cart($param));
    }

    protected function cart($param = array())
    {
        $panier = Panier::getInstance();
        $arrProduit = $panier->getProducts();

        list($sTotalHT, $sTotalTTC) = $panier->getTotalProducts();
        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();
        $shipping = $panier->getShipping();

        $addrShipping = $panier->getAddressShipping();

        $customer = $panier->getCustomer();

        // preparation des données
        $info['client_email'] = (!empty($customer)) ? $customer->client_email : '';
        $info['commande_stotal_ht'] = $sTotalHT;
        $info['commande_stotal_ttc'] = $sTotalTTC;
        $info['commande_stotal_taxe'] = $info['commande_stotal_ttc'] - $info['commande_stotal_ht'];
        $info['commande_port_ht'] = $shipping->port_ht;
        $info['commande_port_ttc'] = $shipping->port_ttc;
        $info['commande_total_ht'] = $aPayerHT;
        $info['commande_total_ttc'] = $aPayerTTC;
        $info['commande_total_taxe'] = $info['commande_total_ttc'] - $info['commande_total_ht'];
        $info['commande_referer'] = ''; // PENDING : lorsque l'application referer
        $info['livraison_ville'] = (!empty($addrShipping)) ? $addrShipping->adresse_ville : '';
        $info['livraison_cp'] = (!empty($addrShipping)) ? $addrShipping->adresse_cp : '';
        $info['livraison_pays'] = (!empty($addrShipping)) ? $addrShipping->pays_nom_nom : '';
        $info['livraison_pays_code'] = (!empty($addrShipping)) ? $addrShipping->pays_iso_code_2 : '';

        $info['produits'] = array();
        foreach ($arrProduit as $product) {
            array_push(
                $info['produits'],
                array(
                    'produit_id' => $product['produit_id'],
                    'produit_ref' => $product['produit_ref'],
                    'produit_nom' => $product['produit_nom_nom'],
                    'prix_ttc' => $product['produit_prix_ttc'],
                    'prix_ht' => $product['produit_prix_ht'],
                    'prix_taxe' => $product['produit_prix_ttc'] - $product['produit_prix_ht'],
                    'qte' => $product['quantite_order'],
                )
            );
        }

        // envoi des données
        return $info;
    }

    /**
     * @return bool|Cookie
     */
    public function initReferer()
    {
        $session = $this->request->getSession();

        $referer = $this->request->get("referer", $this->request->get("utm_source"));

        if (!empty($referer)) {
            if ($session->has('referer') == false || $session->get('referer') != $referer) {
                $session->set('referer', $referer);
            }

            $expire = time() + (3600 * 24 * 30); // valable 30 jours
            return new Cookie('referer', $referer, $expire);
        }

        return false;
    }

    /**
     * @param $type
     * @param $id
     * @param bool $delete
     *
     * @return bool|Cookie
     */
    public function setReferer($type, $id, $delete = false)
    {
        $session = $this->request->getSession();

        if ($session->has('referer')) {
            $query = new Query();
            switch ($type) {
                case 'commande':
                    $query->update('commande')
                        ->value('commande_referer', $session->get('referer'))
                        ->where('commande_id', $id);
                    SiteSQL::getInstance()->query($query->build());
                    break;
            }
        }

        if ($delete == true) {
            if ($session->has('referer')) {
                $session->remove('referer');
            }

            $expire = time() - 3600;
            return new Cookie('referer', '', $expire);
        }

        return false;
    }
}
